/*
 * DS3231.c
 *
 *  Created on: 18 de abr. de 2016
 *      Author: Jorge Saiz Gil
 */

/*-----------------------------------------------------------
 * INCLUDES
 *----------------------------------------------------------*/
#include <Debug.h>
#include <DS3231.h>
#include <I2C.h>
#include <FreeRTOS.h>
#include <semphr.h>

/*-----------------------------------------------------------
 * DECLARACIONES
 *----------------------------------------------------------*/

xSemaphoreHandle xMutexDS3231;

uint8_t I2C_adr; // Variable local para guardar la direccion I2C del DS3231

 /** Bloquea el uso del DS3231 mediante un semaforo de FreeRTOS
*
* @precondition 		La variable xMutexDS3231 tiene que esta definida.
*
* @return NONE
*/
void _Lock_DS3231()
{
	if ( xMutexDS3231 == NULL)
	{
		xMutexDS3231 = xSemaphoreCreateRecursiveMutex();
	}
	xSemaphoreTakeRecursive( xMutexDS3231, portMAX_DELAY );
}

 /** Desbloquea el uso del DS3231 mediante un semaforo de FreeRTOS (Privada)
*
* @precondition		El semaforo tiene que esta creado y cogido.
*
* @return NONE
*/
void _UnLock_DS3231()
{
	xSemaphoreGiveRecursive( xMutexDS3231 );
}

 /** Lee los valores almacenados en el RTC DS3231 y los guarda en un buffer (Privada)
*
* @precondition 		El DS3231 tiene  que estar inicializado
*
* @param buffer  		uint8_t*, puntero a entero que apunta al buffer donde hay que almacenar 
*                  		los valores leidos del DS3231
* 
* @return NONE
*/
void _DS3231_Read(uint8_t* buffer)
{

	_Lock_DS3231();
	I2C0_send(I2C_adr, START_REG, 1);
	I2C0_receive(I2C_adr, (uint8_t*) buffer, 8); //Lee del RTC DS3231, solo necesita 8 registros.
	_UnLock_DS3231();

}

 /** Escribe los valores pasados por parametro en el DS3231 (Privada)
*
* @precondition 		El DS3231 tiene  que estar inicializado
*
* @param buffer  		uint8_t*, puntero a entero que apunta al buffer donde 
*                  		estan los datos a guardar en el DS3231
* 
* @return NONE
*/
void _DS3231_Write(uint8_t* buffer)
{

	_Lock_DS3231();
	buffer[0]=START_REG;
	I2C0_send(I2C_adr, (uint8_t*) buffer, 8); //Escribe en el RTC DS3231
	_UnLock_DS3231();
}

void DS3231_init(uint8_t i2c_port, uint8_t i2c_address, uint32_t i2c_freq)
{
	I2C_adr = i2c_address;
	_Lock_DS3231();
	if (i2c_port == 0)
	{
		I2C0_init(i2c_freq);
		xMutexDS3231 = xSemaphoreCreateMutex();
		DEBUG_Print ( "Modulo DS3231 Inicializado" );
	}
	else
	{
		DEBUG_Print("Error DS3231 puerto I2C %i no implementado.",i2c_port); //version incial solo funciona con puerto I2C0
	}
	_UnLock_DS3231();
}

void DS3231_getTime(Timedate *t)
{
	char buffer[8];
	_Lock_DS3231();
	_DS3231_Read((uint8_t*) buffer);
	t->sec = bcd_to_decimal(buffer[0]);
	t->min = bcd_to_decimal(buffer[1]);
	t->hour = bcd_to_decimal(buffer[2]);
	t->day = bcd_to_decimal(buffer[3]);
	t->date = bcd_to_decimal(buffer[4]);
	t->mon = bcd_to_decimal(buffer[5] & 0x1F); //Limpia bit el asociado a century
	t->year = bcd_to_decimal(buffer[6]);
	_UnLock_DS3231();
}

void DS3231_setTime(Timedate *t)
{
	char buffer[8];
	_Lock_DS3231();
	//buffer[0] reserved to start register address
	buffer[1] = decimal_to_bcd(t->sec);
	buffer[2] = decimal_to_bcd(t->min);
	buffer[3] = decimal_to_bcd(t->hour);
	buffer[4] = decimal_to_bcd(t->day);
	buffer[5] = decimal_to_bcd(t->date);
	buffer[6] = decimal_to_bcd(t->mon);
	buffer[7] = decimal_to_bcd(t->year);

	_DS3231_Write((uint8_t*) buffer);
	_UnLock_DS3231();

}


