/*
 * i2c.c
 *
 *  Created on: 18 de abr. de 2016
 *      Author: Jorge Saiz Gil
 */
/*-----------------------------------------------------------
 * INCLUDES
 *----------------------------------------------------------*/

#include <I2C.h>

/*-----------------------------------------------------------
 * DECLARATIONS
 *----------------------------------------------------------*/ 

static volatile uint8_t  slave_address; // formatted by send or receive
static volatile uint8_t* I2C0Buffer;
static volatile uint32_t I2C0Buffer_len;
static volatile uint32_t num_transferred;
static volatile uint32_t i2c0_busy;

uint32_t ignore_data_nack = 1;

 /** devuelve la direccion de lectura del dispositivo I2C (Privada)
*
* @precondition   NONE      
*
* @param address       uint8_t, Entero con la direccion de memoria del disp. I2C.
* 
* @return Entero con la direccion de lectura del disp. I2C, uint8_t
*/
static inline uint8_t _to_read_address(uint8_t address)
{
    return (address << 1) | 0x01; //define read address
}

 /** dDevuelve la direccion de escritura del dispositivo I2C (Privada)
*
* @precondition   NONE      
*
* @param address       uint8_t, Entero con la direccion del disp. I2C
* 
* @return Entero con la direccion de escritura del disp. I2C, uint8_t
*/
static inline uint8_t _to_write_address(uint8_t address)
{
    return (address << 1); //define write address
} 

 
void I2C0_init(uint32_t i2c_freq)
{
    uint32_t pclk, fdiv;
    
    // setup initial state
    i2c0_busy = 0;
    I2C0Buffer_len = 0;
    num_transferred = 0;
 
    // give power to the I2C hardware
    LPC_SC->PCONP |= (1 << 7);
 
    // set PIO0.27 and PIO0.28 to I2C0 SDA and SCK
    LPC_PINCON->PINSEL1 &= ~0x03C00000;
    LPC_PINCON->PINSEL1 |=  0x01400000;
 
    // set peripheral clock selection for I2C0
    LPC_SC->PCLKSEL0 &= ~(3 << 14); // clear bits
    LPC_SC->PCLKSEL0 |=  (1 << 14); // set to "01" (full speed)
    pclk = SystemCoreClock;
 
    // clear all flags
    LPC_I2C0->I2CONCLR = AA | SI | STO | STA | I2EN;
 
    // determine the frequency divider and set corresponding registers
    //  this makes a 50% duty cycle
    fdiv = pclk / i2c_freq;
    LPC_I2C0->I2SCLL = fdiv >> 1; // fdiv / 2
    LPC_I2C0->I2SCLH = fdiv - (fdiv >> 1); // compensate for odd dividers
 
    // install interrupt handler
    NVIC_EnableIRQ(I2C0_IRQn);
 
    // enable the I2C (master only)
    LPC_I2C0->I2CONSET = I2EN;
}
 
uint32_t I2C0_send(uint8_t address, uint8_t* I2C_Send_Buffer, uint32_t length)
{
    // check if i2c0 is busy
    if (i2c0_busy)
	{
		return 0;
	}
 
    // set to status to 'busy'
    i2c0_busy = 1;
 
    // setup pointers
    slave_address = _to_write_address(address);
    I2C0Buffer = I2C_Send_Buffer;
    I2C0Buffer_len = length;
    num_transferred = 0;
 
    // trigger a start condition
    LPC_I2C0->I2CONSET = STA;
 
    // wait for transmission
    while (i2c0_busy);
 
    // return bytes transferred
    return num_transferred;
}
 
uint32_t I2C0_receive(uint8_t address, uint8_t* I2C_Read_Buffer, uint32_t length)
{
    // check software FSM
    if (i2c0_busy)
	{
			return 0;
	}
  
    // set to status to 'busy'
    i2c0_busy = 1;
 
    // setup pointers
    slave_address = _to_read_address(address);
    I2C0Buffer = I2C_Read_Buffer;
    I2C0Buffer_len = length;
    num_transferred = 0;
 
    // trigger a start condition
    LPC_I2C0->I2CONSET = STA;
 
    // wait for transmission
    while (i2c0_busy);
 
    // return bytes transferred
    return num_transferred;
}
 
void I2C0_IRQHandler(void) 
{
    // get reason for interrupt
    uint8_t status = LPC_I2C0->I2STAT;
 
    // ignore data nack when control is true
    if ((status == 0x30) && (ignore_data_nack))
	{
		status = 0x28;
	}
    
    switch(status) {
 
    // Int: start condition has been transmitted
    // Do:  send SLA+R or SLA+W
    case 0x08:
        LPC_I2C0->I2DAT = slave_address; // formatted by send or receive
        LPC_I2C0->I2CONCLR = STA | SI;
        break;
 
    // Int: repeated start condition has been transmitted
    // Do:  send SLA+R or SLA+W
    //case 0x10:
    //    LPC_I2C0->I2DAT = slave_address;
    //    LPC_I2C0->I2CONCLR = STA | SI;
    //    break;
 
    // Int: SLA+W has been transmitted, ACK received
    // Do:  send first byte of buffer if available
    case 0x18:
        if (num_transferred < I2C0Buffer_len) {
            LPC_I2C0->I2DAT = I2C0Buffer[0];
            LPC_I2C0->I2CONCLR = STO | STA | SI;
        }
        else {
            LPC_I2C0->I2CONCLR = STA | SI;
            LPC_I2C0->I2CONSET = STO;
        }
        break;
 
    // Int: SLA+W has been transmitted, NACK received
    // Do:  stop!
    case 0x20:
        LPC_I2C0->I2CONCLR = STA | SI;
        LPC_I2C0->I2CONSET = STO;
        num_transferred = 0xFFFFFFFF;
        i2c0_busy = 0;
        break;
 
    // Int: data byte has been transmitted, ACK received
    // Do:  load next byte if available, else stop
    case 0x28:
        num_transferred++;
        if (num_transferred < I2C0Buffer_len) {
            LPC_I2C0->I2DAT = I2C0Buffer[num_transferred];
            LPC_I2C0->I2CONCLR = STO | STA | SI;
        }
        else {
            LPC_I2C0->I2CONCLR = STA | SI;
            LPC_I2C0->I2CONSET = STO;
            i2c0_busy = 0;
        }
        break;
 
    // Int: data byte has been transmitted, NACK received
    // Do:  stop!
    case 0x30:
        LPC_I2C0->I2CONCLR = STA | SI;
        LPC_I2C0->I2CONSET = STO;
        i2c0_busy = 0;
        break;
 
    // Int: arbitration lost in SLA+R/W or Data bytes
    // Do:  release bus
    case 0x38:
        LPC_I2C0->I2CONCLR = STO | STA | SI;
        i2c0_busy = 0;
        break;
 
    // Int: SLA+R has been transmitted, ACK received
    // Do:  determine if byte is to be received
    case 0x40:
        if (num_transferred < I2C0Buffer_len) {
            LPC_I2C0->I2CONCLR = STO | STA | SI;
            LPC_I2C0->I2CONSET = AA;
        }
        else {
            LPC_I2C0->I2CONCLR = AA | STO | STA | SI;
        }
        break;
 
    // Int: SLA+R has been transmitted, NACK received
    // Do:  stop!
    case 0x48:
        LPC_I2C0->I2CONCLR = STA | SI;
        LPC_I2C0->I2CONSET = STO;
        num_transferred = 0xFFFFFFFF;
        i2c0_busy = 0;
        break;
 
    // Int: data byte has been received, ACK has been returned
    // Do:  read byte, determine if another byte is needed
    case 0x50:
        I2C0Buffer[num_transferred] = LPC_I2C0->I2DAT;
        num_transferred++;
        if (num_transferred < I2C0Buffer_len) {
            LPC_I2C0->I2CONCLR = STO | STA | SI;
            LPC_I2C0->I2CONSET = AA;
        }
        else {
            LPC_I2C0->I2CONCLR = AA | STO | STA | SI;
        }
        break;
 
    // Int: data byte has been received, NACK has been returned
    // Do:  transfer is done, stop.
    case 0x58:
        LPC_I2C0->I2CONCLR = STA | SI;
        LPC_I2C0->I2CONSET = STO;
        i2c0_busy = 0;
        break;
 
    // something went wrong, trap error
    default:
        while (1); // flash a LED or something??
        break;
 
    }
}
