/*
 * UARTprint.c
 *
 *  Created on: 18/04/2016
 *      Author: Jorge Saiz Gil
 */

/*-----------------------------------------------------------
 * INCLUDES
 *----------------------------------------------------------*/

#include <LPC17xx.h>
#include <FreeRTOS.h>
#include <semphr.h>

#include <Config.h> 
#include <WIFLY.h>
#include <Timer.h>
#include <type.h>
#include <Debug.h>
#include <UARTprint.h>
#include <uart.h>

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>



xSemaphoreHandle xMutexWifly;

uint32_t WIFLYPort; //variable local para almacenar el puerto UART del modulo wifly
uint8_t SleepStatus;
char response[BUFSIZE];

/** Bloquea el uso del WIFLY mediante un semaforo de FreeRTOS (Privada)
*
* @precondition 		La variable xMutexWifly tiene que estar definida.
*
* @return NONE
*/
void _Lock_Wifly()
{
	if ( xMutexWifly == NULL)
	{
		xMutexWifly = xSemaphoreCreateRecursiveMutex();
	}
	xSemaphoreTakeRecursive( xMutexWifly, portMAX_DELAY );
}

/** Desbloquea el uso del WIFLY mediante un semaforo de FreeRTOS (Privada)
*
* @precondition 		La variable xMutexWifly tiene que estar definida.
*
* @return NONE
*/
void _UnLock_Wifly()
{
	xSemaphoreGiveRecursive( xMutexWifly );
}

/** Realiza un delay mediante contadores, no utiliza el delay de tareas (Privada)
*
* @precondition 		NONE
*
* @param ms 			uint32_t, Entero con los ms a esperar 	
* 
* @return NONE
*/
void _delayWifly(uint32_t ms)
{
	uint32_t i;
	for(i=0; i<(((uint32_t)7140)*ms);i++); // 7140 aproximadamente son 1 ms
}

/** Resetea el modulo Wifly (Privada)
*
* @precondition 		El modulo wilfy debe estar inicializado
*
* @return NONE
*/
void _WIFLY_reset()
{
	GPIO_PinFunction(WIFLY_RESET_PIN, PINSEL_FUNC_0); /// Configuramos el pin GPIO para el RESET del wifly
	GPIO_PinDirection(WIFLY_RESET_PIN, 1);	// GPIO de salida.
	GPIO_PinWrite(WIFLY_RESET_PIN, 0); // enviamos 0
	_delayWifly(1);
	GPIO_PinWrite(WIFLY_RESET_PIN, 1); // enviamos 1

}

/** Hace que el modulo Wifly entre en modo COMMAND (Privada)
*
* @precondition 		El modulo wilfy debe estar inicializado
*
* @return NONE
*/
uint8_t _WIFLY_EnterCMD()
{
	uint8_t ret = FALSE;
	memset(response,0,BUFSIZE);

	_Lock_Wifly(); //cogemos mutex
	UARTPrint_Text(WIFLYPort, (char*)"$$$");
	ret = UART_CheckRead(WIFLYPort,(char*)response ,(char *)"CMD",SIMPLE_TIMEOUT);
#ifdef DEBUG_MODE
	if (ret == TRUE)
	{
		DEBUG_Print("WIFLY: Entrando en modo CMD");
	}
	else
	{
		DEBUG_Print("WIFLY: Problema entrando en modo CMD");
	}
#endif
	_UnLock_Wifly();
	return ret;
}

/** Configura el servidor NTP donde se realizaran las peticiones
*	necesario para utilizar la funcionalidad interna de sinzeonizacion de tiempo (Privada)
*
* @precondition 		El modulo wilfy debe estar inicializado
*
 @param serverIP 		char*, Puntero a char con la ip del servidor NTP
*
* @return NONE
*/
uint8_t _WIFLY_Set_NTPServer(char* serverIP)
{
	uint8_t ret = FALSE;
	memset(response,0,BUFSIZE);
	_Lock_Wifly(); //cogemos mutex
	UARTPrint_Text_LF(WIFLYPort, "set time address %s",serverIP);
	ret = UART_CheckRead(WIFLYPort,response ,(char *) "AOK",SIMPLE_TIMEOUT);
#ifdef DEBUG_MODE
	if (ret == TRUE)
	{
		DEBUG_Print("WIFLY: Servidor NTP %s guardado en el modulo Wifly.",serverIP);
	}
	else
	{
		DEBUG_Print("WIFLY: Problema definiendo el servidor NTP");
	}
#endif
	_UnLock_Wifly();
	return ret;
}

/** Configura el servidor donde se realizaran las peticiones (Privada)
*
* @precondition 		El modulo wilfy debe estar inicializado
*
 @param host 			char*, Puntero a char donde esta la ip del servidor
*
* @return NONE
*/
uint8_t _WIFLY_Set_RemoteHost(char* host)
{
	uint8_t ret = FALSE;
	memset(response,0,BUFSIZE);
	_Lock_Wifly(); //cogemos mutex
	UARTPrint_Text_LF(WIFLYPort, "set ip host %s",host);
	ret = UART_CheckRead(WIFLYPort,response ,(char *) "AOK",SIMPLE_TIMEOUT);
#ifdef DEBUG_MODE
	if (ret == TRUE)
	{
		DEBUG_Print("WIFLY: Servidor %s guardado.",host);
	}
	else
	{
		DEBUG_Print("WIFLY: Problema guardando servidor.");
	}
#endif
	_UnLock_Wifly();
	return ret;
}
/** Configura el puerto del servidor remoto (Privada)
*
* @precondition 		El modulo wilfy debe estar inicializado
*
 @param port 			char*, Punto a char donde esta el valor del puerto
*
* @return NONE
*/
uint8_t _WIFLY_Set_RemotePort(char* port)
{
	uint8_t ret = FALSE;
	memset(response,0,BUFSIZE);
	_Lock_Wifly(); //cogemos mutex
	UARTPrint_Text_LF(WIFLYPort, "set ip remote %s",port);
	ret = UART_CheckRead(WIFLYPort,response ,(char *) "AOK",SIMPLE_TIMEOUT);
#ifdef DEBUG_MODE
	if (ret == TRUE)
	{
		DEBUG_Print("WIFLY: Puerto %s guardado.",port);
	}
	else
	{
		DEBUG_Print("WIFLY: Problema guardando puerto.");
	}
#endif
	_UnLock_Wifly();
	return ret;
}

/** Configura el SSID en el modulo Wifly (Privada)
*
* @precondition 		El modulo wilfy debe estar inicializado
*
 @param ssid 			char*, Punto a char donde esta el valor del SSID
*
* @return NONE
*/
uint8_t _WIFLY_Set_SSID(char* ssid)
{
	uint8_t ret = FALSE;
	memset(response,0,BUFSIZE);
	_Lock_Wifly(); //cogemos mutex
	UARTPrint_Text_LF(WIFLYPort, "set wlan ssid %s",ssid);
	ret = UART_CheckRead(WIFLYPort,response ,(char *) "AOK",SIMPLE_TIMEOUT);
#ifdef DEBUG_MODE
	if (ret == TRUE)
	{
		DEBUG_Print("WIFLY: SSID %s guardado en el modulo Wifly.", ssid);
	}
	else
	{
		DEBUG_Print("WIFLY: Problema definiendo el SSID");
	}
#endif
	_UnLock_Wifly();
	return ret;
}

/** Configura la contraseña de la red para redes WEP (Privada)
*
* @precondition 		El modulo wilfy debe estar inicializado
*
 @param key 			char*, Punto a char donde esta el valor de la contraseña
*
* @return NONE
*/
uint8_t _WIFLY_Set_Key(char* key)
{
	uint8_t ret = FALSE;
	memset(response,0,BUFSIZE);
	_Lock_Wifly(); //cogemos mutex
	UARTPrint_Text_LF(WIFLYPort, "set wlan key %s", key);
	ret = UART_CheckRead(WIFLYPort,response ,(char *) "AOK",SIMPLE_TIMEOUT);
#ifdef DEBUG_MODE
	if (ret == TRUE)
	{
		DEBUG_Print("WIFLY: Contraseña WEP de la red guardada en el modulo Wifly");
	}
	else
	{
		DEBUG_Print("WIFLY: Problema definiendo la contraseña WEP");
	}
#endif
	_UnLock_Wifly();
	return ret;
}

/** Configura la contraseña de la red para redes WPA (Privada)
*
* @precondition 		El modulo wilfy debe estar inicializado
*
 @param phrase 			char*, Punto a char donde esta el valor de la contraseña
*
* @return NONE
*/
uint8_t _WIFLY_Set_Phrase(char* phrase)
{
	uint8_t ret = FALSE;
	memset(response,0,BUFSIZE);
	_Lock_Wifly(); //cogemos mutex
	UARTPrint_Text_LF(WIFLYPort, "set wlan phrase %s", phrase);
	ret = UART_CheckRead(WIFLYPort,response ,(char *) "AOK",SIMPLE_TIMEOUT);
#ifdef DEBUG_MODE
	if (ret == TRUE)
	{
		DEBUG_Print("WIFLY: Contraseña WPA de la red guardada en el modulo Wifly");
	}
	else
	{
		DEBUG_Print("WIFLY: Problema definiendo la contraseña WPA");
	}
#endif
	_UnLock_Wifly();
	return ret;
}

/** Envia comando parado por parametro al modulo Wifly, 
*   util para comandos que no necesitan parametros (Privada)
*
* @precondition 		El modulo wilfy debe estar inicializado
*
* @param command			char*, Punto a char donde esta el valor del comando
* @param value				char*, Punto a char donde esta el valor de respuesta esperado
* @param Delay				uint32_t, Entero con el tiempo a esperar
*
* @return NONE
*/
uint8_t _WIFLY_Send_Command(char* command,char* value, uint32_t Delay)
{
	uint8_t ret = FALSE;
	memset(response,0,BUFSIZE);
	_Lock_Wifly(); //cogemos mutex
	UARTPrint_Text_LF(WIFLYPort, command);
	ret = UART_CheckRead(WIFLYPort,response ,value,Delay);
#ifdef DEBUG_MODE
	if (ret == TRUE)
	{
		DEBUG_Print("WIFLY: %s. Comando procesado correctamente.",command);
	}
	else
	{
		DEBUG_Print("WIFLY: %s. Problema procesando el comnando.",command);
	}
#endif
	_UnLock_Wifly(); //dejamos mutex
	return ret;
}

void WIFLY_Init( uint32_t portNum, uint32_t Baudrate )
{
	_Lock_Wifly(); //cogemos mutex
	WIFLYPort = portNum;
	UART_Clear(WIFLYPort);
	UARTInit(WIFLYPort, Baudrate); // init wifly uart
	_WIFLY_reset(); //reset del modulo Wifly
	_delayWifly(1000);
#ifdef DEBUG_MODE
	DEBUG_Print ( "UART Wifly online" );
#endif
	SleepStatus= FALSE;
	_UnLock_Wifly();
}

void WIFLY_Connect_WLAN(char* ssid, char* pwd, uint8_t mode)
{
	_Lock_Wifly();
#ifdef DEBUG_MODE
	DEBUG_Print("--------------------------------------");
	DEBUG_Print("WIFLY: Intentando conectar a Internet");
	DEBUG_Print("--------------------------------------");
#endif
	_WIFLY_EnterCMD();
	_WIFLY_Send_Command("factory RESET","Factory Defaults",SIMPLE_TIMEOUT); // reset to factory config
	_WIFLY_Send_Command("reboot","*Reboot*",LARGE_TIMEOUT);
	_WIFLY_EnterCMD();

	_WIFLY_Set_SSID((char*) ssid); //set SSID
	switch(mode)
	{
		case 0: //WPA
			_WIFLY_Set_Phrase((char*) pwd);
			break;
		case 1: //WEP
			_WIFLY_Set_Key((char*) pwd);
			break;
	}
	_WIFLY_Send_Command("set wlan join 1","AOK",SIMPLE_TIMEOUT); // se intenta autoconetar cuando se inicia o despierta
	_WIFLY_Send_Command("set sys trigger 1","AOK",SIMPLE_TIMEOUT); //se despierta cuando recibe algun dato por UART
	_WIFLY_Send_Command("set sys printlvl 0x4000","AOK",SIMPLE_TIMEOUT); //commandos amigables con mcus
	_WIFLY_Send_Command("set ip proto 1","AOK",SIMPLE_TIMEOUT); // activamos protcolo UDP
	_WIFLY_Send_Command("save","Storing in config",SIMPLE_TIMEOUT);
	_WIFLY_Send_Command("join","Associated!",LARGE_TIMEOUT);
	_WIFLY_Send_Command("exit","EXIT",SIMPLE_TIMEOUT);
#ifdef DEBUG_MODE
	DEBUG_Print("--------------------------------------");
	DEBUG_Print("\r\n");
	DEBUG_Print("--------------------------------------");
#endif
	_UnLock_Wifly();
}

void WIFLY_ConfigNTP(char* server,char* port)
{
	_Lock_Wifly();
#ifdef DEBUG_MODE
	DEBUG_Print("--------------------------------------");
	DEBUG_Print("WIFLY: Configurando parametros:");
	DEBUG_Print("--------------------------------------");;
#endif
	_WIFLY_EnterCMD();
	//_WIFLY_Set_NTPServer(server); //ACTIVAR PARA USAR FUNCIONALIDAD INTERNA DEL WILFY PARA OBTENER EL TIEMPO
	_WIFLY_Set_RemoteHost(server);
	_WIFLY_Set_RemotePort(port);
	_WIFLY_Send_Command("save","Storing in config",SIMPLE_TIMEOUT);
	_WIFLY_Send_Command("exit","EXIT",SIMPLE_TIMEOUT);

#ifdef DEBUG_MODE
	DEBUG_Print("--------------------------------------");
	DEBUG_Print(" ");
#endif
	_UnLock_Wifly();
}

uint32_t WIFLY_GetTime()
{
	uint32_t int_newTimestamp = 0;
	memset(response,0,BUFSIZE);
	char char_newTimestamp[10];
	char* pos;
	uint8_t timeNotSet = FALSE;
	uint8_t i=0;
	_Lock_Wifly();
#ifdef DEBUG_MODE
	DEBUG_Print("--------------------------------------");
	DEBUG_Print("WIFLY: Obteniendo tiempo desde Wifly");
	DEBUG_Print("--------------------------------------");
#endif
	_WIFLY_EnterCMD();
	///************ Iniciamos el TIMER ***************************//
	Timer_Start(TIMER_NUM0);
	//*************************************************************//
	_WIFLY_Send_Command("time","AOK",SIMPLE_TIMEOUT);
	UARTPrint_Text_LF(WIFLYPort,"show t t");
	timeNotSet = UART_CheckRead(WIFLYPort,response ,(char*)"NOT SET",SIMPLE_TIMEOUT);
	if (!timeNotSet)
	{
		pos = strstr(response,"RTC=");
		strncpy(char_newTimestamp,pos+4,10);
		char_newTimestamp[10]='\0';
#ifdef DEBUG_MODE
		DEBUG_Print("WIFLY: Tiempo obtenido %s",char_newTimestamp);
#endif
		//convierte la cadena newTimestamp en un entero.
		for(i=0;i<10;i++)
		{
			int_newTimestamp= int_newTimestamp*10 + (char_newTimestamp[i]-'0');
		}
	}
	else
	{
		int_newTimestamp = 0;
#ifdef DEBUG_MODE
		DEBUG_Print("WIFLY: Problema obteniendo Tiempo");
#endif
	}
	_WIFLY_Send_Command("exit","EXIT",SIMPLE_TIMEOUT);
#ifdef DEBUG_MODE
	DEBUG_Print("--------------------------------------");
	DEBUG_Print(" ");
#endif
	_UnLock_Wifly();

	return int_newTimestamp;
}

uint8_t WIFLY_CheckConnection()
{
	uint8_t ret;
	memset(response,0,BUFSIZE);

	_Lock_Wifly();
#ifdef DEBUG_MODE
	DEBUG_Print("--------------------------------------");
	DEBUG_Print("WIFLY: Conprobando conexion a internet");
	DEBUG_Print("--------------------------------------");;
#endif

	if (SleepStatus)
	{
		UARTPrint_Text(WIFLYPort,"1"); //despierta el modulo si esta dormido.
		delayMs(LARGE_TIMEOUT);
	}

	_WIFLY_EnterCMD();
	UARTPrint_Text_LF(WIFLYPort,"show connection");
	ret = UART_CheckRead(WIFLYPort,response ,(char*)"8730",LONG_TIMEOUT);
	if ( ret == TRUE )
	{
		ret = TRUE;
	}
	else
	{
		ret = FALSE;
	}
	_WIFLY_Send_Command("exit","EXIT",SIMPLE_TIMEOUT);
#ifdef DEBUG_MODE
	DEBUG_Print("--------------------------------------");
	DEBUG_Print(" ");
#endif
	_UnLock_Wifly();

	return ret;
}

uint32_t Wifly_NTP_request()
{
	uint32_t sizeResp;
	uint32_t Recieve_TS_sec;
	char UDP_Send_Buffer[48];
	uint8_t NTP_Packet[48];
	uint32_t UNIXepoch = 2208988800u;	// 1/1/1970 00:00
	memset(UDP_Send_Buffer,0,48);

	_Lock_Wifly();

#ifdef DEBUG_MODE
	DEBUG_Print("--------------------------------------");
	DEBUG_Print("WIFLY: Sync. tiempo desde servidor NTP");
	DEBUG_Print("--------------------------------------");
#endif
	// Initialize values needed to form NTP request
	UDP_Send_Buffer[0] = 0x23;//0b0011011; LI=0, Version=4, Mode=client
	UDP_Send_Buffer[1] = 0x00; // Stratum, or type of clock
	UDP_Send_Buffer[2] = 0x00; // Polling Interval
	UDP_Send_Buffer[3] = 0x00; // Peer Clock Precision

	///************ Iniciamos el TIMER ***************************//
			Timer_Start(TIMER_NUM0);
	//*************************************************************//
	UARTPrint_PacketUDP(WIFLYPort,(char *)UDP_Send_Buffer,48);
	Recieve_TS_sec = 0;
	if((sizeResp=UART_Read_NTP_Packet(WIFLYPort, NTP_Packet, 2000)) == 48 )
	{
		Recieve_TS_sec = NTP_Packet[40] << 24 | NTP_Packet[41] << 16 | NTP_Packet[42] << 8 | NTP_Packet[43]; // parte de segundos del TS
		Recieve_TS_sec -= UNIXepoch; // se ajusta a UNIX time.
#ifdef DEBUG_MODE
		DEBUG_Print("WIFLY: Tiempo obtenido %i",Recieve_TS_sec);
#endif
	}
	else
	{
#ifdef DEBUG_MODE
		DEBUG_Print("WIFLY: Problema obteniendo Tiempo");
#endif
	}
#ifdef DEBUG_MODE
	DEBUG_Print("--------------------------------------");
	DEBUG_Print(" ");
#endif
	_UnLock_Wifly();
	
	return Recieve_TS_sec;
}

uint8_t Wifly_SleepModule()
{
	_Lock_Wifly();
#ifdef DEBUG_MODE
	DEBUG_Print("--------------------------------------");
	DEBUG_Print("WIFLY: Pasando a modo en reposo");
	DEBUG_Print("--------------------------------------");
#endif

	_WIFLY_EnterCMD();
	SleepStatus = _WIFLY_Send_Command("sleep","sleep",SIMPLE_TIMEOUT);

#ifdef DEBUG_MODE
	DEBUG_Print("--------------------------------------");
	DEBUG_Print(" ");
#endif
	_UnLock_Wifly();
	return SleepStatus;
}
