
#include <LPC17xx.h>
#include <FreeRTOS.h>
#include <task.h>
#include <queue.h>
#include <semphr.h>

#include <UARTprint.h>
#include <uart.h>
#include <WIFLY.h>
#include <DS3231.h>
#include <I2C.h>
#include <GPIO.h>
#include <LedDisplay.h>
#include <Timer.h>

#include <TimeTools.h>
#include <string.h>
#include <type.h>
#include <Config.h>


/*-----------------------------------------------------------
 * DECLARACIONES
 *----------------------------------------------------------*/

xQueueHandle xQueueReceiveTime; //Cola donde se almacenara el tiempo sincronizado

static void WordClock_init(); // Funcion que inicializa todos los componentes del dispositivo.

//Tareas
static void Task_ShowTimeOnDisplay( void *pvParameters ); // Tarea que muestra el tiempo en el panel
static void Task_UpdateTimeRTC( void *pvParameters ); 	//Tarea que actualizar el tiempo en el RTC
static void Task_SyncTimeOnline( void *pvParameters );	//Tara que sincroniza el tiempo online


/*----------------------------------------------------------*/

int main (void)
{
	WordClock_init();
	DEBUG_Print("***************************************");
	DEBUG_Print("   Componentes Inicializados");
	DEBUG_Print("***************************************");

	xQueueReceiveTime = xQueueCreate( 1, sizeof( uint32_t ) );
	xTaskCreate( Task_UpdateTimeRTC, "Actualiza el tiempo en el RTC si es necesario", 240, NULL, 2, NULL );
	xTaskCreate( Task_SyncTimeOnline, "Obtiene el tiempo desde internet si hay conexion", 240, NULL, 1, NULL );
	xTaskCreate( Task_ShowTimeOnDisplay, "Controla la hora que se muestra en el panel de leds", 240, NULL, 1, NULL );
			/* Start the tsks running. */
	vTaskStartScheduler();
	for( ;; );
}


static void Task_UpdateTimeRTC( void *pvParameters )
{
	portBASE_TYPE xStatus;
	uint32_t xReceivedTime;
	Timedate tdReceivedTime;
	const portTickType xTicksToWait = 10 / portTICK_RATE_MS;

	for( ;; )
	{
		xStatus = xQueueReceive( xQueueReceiveTime, &xReceivedTime,xTicksToWait );
		if (xStatus == pdPASS )
		{
			Time_TimestamptoTimedate(xReceivedTime, &tdReceivedTime);
			DS3231_setTime(&tdReceivedTime);
			DEBUG_Print("***************************************");
			DEBUG_Print("   Tiempo actualizado en el RTC:");
			DEBUG_Print("   %i",xReceivedTime);
			DEBUG_Print("   %02d:%02d:%02d %02d/%02d/%02d  day: %d",tdReceivedTime.hour,tdReceivedTime.min,tdReceivedTime.sec,tdReceivedTime.date,tdReceivedTime.mon,tdReceivedTime.year,tdReceivedTime.day);
			DEBUG_Print("***************************************");
			DEBUG_Print(" ");
		}
	}
}

static void Task_SyncTimeOnline( void *pvParameters )
{
	uint32_t ts = 0;
	uint32_t delayUs = 0;
	uint32_t check_freq = WIFLY_SYNC_TIME_FREQ;
	portTickType xTicksToWait = 100 / portTICK_RATE_MS;

	delayMs(LONG_TIMEOUT); //esperamos a que arranque el modulo Wifly
	WIFLY_Connect_WLAN("AP-SAGRA","336c336e407831306a30726733", 1);
	WIFLY_ConfigNTP(WIFLY_NTP_SERVER,WIFLY_NTP_PORT);
	for( ;; )
	{

		if (WIFLY_CheckConnection())
		{
			DEBUG_Print("***************************************");
			DEBUG_Print("    Estado de conexion a internet");
			DEBUG_Print("              ONLINE");
			DEBUG_Print("***************************************");
			DEBUG_Print(" ");

			ts = Wifly_NTP_request();
			//ts = WIFLY_GetTime(); //devuelve 0 si ha habido algun problema

			if (ts > 0)
			{
				DEBUG_Print("***************************************");
				DEBUG_Print("Tiempo actualizado a traves de Internet");
				DEBUG_Print("***************************************");
				DEBUG_Print(" ");

				/****************** TIMER STOP COUNTING *******************************************/
				ts = Time_adjustTimeZoneDST(ts);
				delayUs = TIMER_GetTime(TIMER_NUM0);
				ts = Time_adjunstSyncDelay(ts,delayUs);
				xQueueSendToBack( xQueueReceiveTime, &ts, xTicksToWait );
				Timer_Stop(TIMER_NUM0);
				/**************************************************************************/
				check_freq = WIFLY_SYNC_TIME_FREQ;

			}
			else
			{
				check_freq = WIFLY_RESYNC_TIME_FREQ; // como no ha sincronizado el tiempo bajamos el tiempo de reintento.
			}
			if (Wifly_SleepModule())
			{
				DEBUG_Print("***************************************");
				DEBUG_Print("      Modulo WIFLY en reposo");
				DEBUG_Print("***************************************");
				DEBUG_Print(" ");
			}

		}
		else
		{
			DEBUG_Print("***************************************");
			DEBUG_Print("    Estado de conexion a internet");
			DEBUG_Print("              OFFLINE");
			DEBUG_Print("***************************************");
			DEBUG_Print(" ");
			WIFLY_Connect_WLAN("AP-SAGRA","336c336e407831306a30726733", 1);
			WIFLY_ConfigNTP(WIFLY_NTP_SERVER,WIFLY_NTP_PORT);
			check_freq = WIFLY_RESYNC_TIME_FREQ;


		}

		delayMs(check_freq);
	}
}
static void Task_ShowTimeOnDisplay( void *pvParameters )
{
	uint8_t actualMin=60;
	Timedate xtime;
	Timedate* ptime = &xtime;

	for( ;; )
	{
		DS3231_getTime(ptime);
		if (ptime->min != actualMin )
		{
			LedDisplay_showTime(ptime);
			DEBUG_Print("***************************************");
			DEBUG_Print("    Tiempo actualizado en el panel");
			DEBUG_Print("***************************************");
			DEBUG_Print(" ");
			actualMin = ptime->min;


		}
		DEBUG_Print("Hora actual: %02d:%02d:%02d %02d/%02d/%02d  day: %d",xtime.hour,xtime.min,xtime.sec,xtime.date,xtime.mon,xtime.year,xtime.day);
		delayMs(1000);
	}
}



static void WordClock_init()
{
	LedDisplay_init(); //init GPIOs for leddisplay
	DEBUG_Init(UART_DEBUG, UART_DEBUG_BR); //init Debug UART
	WIFLY_Init(WIFLY_UART,WIFLY_BAUDRATE); // init wifly
	DS3231_init(DS3231_I2C_PORT,DS3231_ADDRESS,I2C_MODE_400kbps);
	Timer_init(TIMER_NUM0);
}


